@@ -3,15 +3,16 @@
 import os
 import sys
 
-sys.path.insert(0, ".")
-sys.path.insert(0, os.path.abspath('../'))
+dir_ = os.path.dirname(os.path.realpath(__file__))
+sys.path.insert(0, dir_)
+sys.path.insert(0, os.path.abspath(os.path.join(dir_, "..")))
 from quodlibet import const
 
 extensions = ['sphinx.ext.autodoc', 'ext']
 source_suffix = '.rst'
 master_doc = 'index'
 project = 'Ex Falso / Quod Libet'
-copyright = 'The Quod Libet Devs'
+copyright = "2004-2015 %s and more" % ", ".join(const.MAIN_AUTHORS)
 version = ".".join(const.VERSION.rsplit(".")[:2])
 release = const.VERSION
 if release.endswith(".-1"):
@@ -19,12 +20,12 @@ if release.endswith(".-1"):
 exclude_patterns = ['_build', '_rtd_theme_repo', 'README.rst']
 html_theme = "haiku"
 html_title = "%s (%s)" % (project, version)
-bug_url_template = "http://code.google.com/p/quodlibet/issues/detail?id=%s"
+bug_url_template = "https://github.com/quodlibet/quodlibet/issues/%s"
 
 RTD_NEW_THEME = True
 
 # on a stable branch which isn't a release
-if const.BRANCH_NAME != "default" and const.VERSION_TUPLE[-1] == -1:
+if const.BRANCH_NAME != "master" and const.VERSION_TUPLE[-1] == -1:
     rst_prolog = """
 
 .. note::
