@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2012 Christoph Reiter
 #
 # This software and accompanying documentation, if any, may be freely
@@ -8,7 +9,6 @@
 import os
 import subprocess
 
-from distutils.util import change_root
 from distutils.core import Command
 
 
@@ -30,45 +30,25 @@ def update_icon_cache(*args):
     return True
 
 
-class build_icon_cache(Command):
-    """Update the icon theme cache"""
-
-    user_options = []
-
-    def initialize_options(self):
-        pass
-
-    def finalize_options(self):
-        pass
-
-    def run(self):
-        if not update_icon_cache('-f', 'quodlibet/images/hicolor'):
-            print "WARNING: gtk-update-icon-cache missing"
-
-
 class install_icons(Command):
     """Copy app icons to hicolor/pixmaps and update the global cache"""
 
     user_options = []
-    root = None
-    prefix = None
+    install_dir = None
 
     def initialize_options(self):
         self.outfiles = []
 
     def finalize_options(self):
         self.set_undefined_options('install',
-                                   ('root', 'root'),
-                                   ('install_base', 'prefix'))
+                                   ('install_data', 'install_dir'))
 
     def get_outputs(self):
         return self.outfiles
 
     def run(self):
         # install into hicolor icon theme
-        basepath = os.path.join(self.prefix, 'share', 'icons', 'hicolor')
-        if self.root is not None:
-            basepath = change_root(self.root, basepath)
+        basepath = os.path.join(self.install_dir, 'share', 'icons', 'hicolor')
 
         local = os.path.join("quodlibet", "images", "hicolor")
 
@@ -86,9 +66,6 @@ class install_icons(Command):
         update_icon_cache(basepath)
 
         # install png versions to /usr/share/pixmaps
-        basepath = os.path.join(self.prefix, 'share', 'pixmaps')
-        if self.root is not None:
-            basepath = change_root(self.root, basepath)
-
+        basepath = os.path.join(self.install_dir, 'share', 'pixmaps')
         out = self.copy_tree(png, basepath)
         self.outfiles.extend(out)
