@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2007 Joe Wreschnig
 #
 # This software and accompanying documentation, if any, may be freely
@@ -14,29 +15,30 @@ gettext message catalogs.
 import os
 import glob
 from subprocess import Popen, PIPE
+from tempfile import mkstemp
+import shutil
 
 from distutils.dep_util import newer
-from distutils.util import change_root
 from distutils.spawn import find_executable
 from distutils.core import Command
 
+from . import gettextutil
+
 
 class po_stats(Command):
+
     description = "Show translation statistics"
     user_options = []
-    build_base = None
 
     def initialize_options(self):
         pass
 
     def finalize_options(self):
-        self.po_package = self.distribution.po_package
         self.po_directory = self.distribution.po_directory
         self.po_files = glob.glob(os.path.join(self.po_directory, "*.po"))
-        self.set_undefined_options('build', ('build_base', 'build_base'))
 
     def run(self):
-        self.run_command("build_mo")
+        self.run_command("update_po")
         res = []
         for po in self.po_files:
             language = os.path.basename(po).split(".")[0]
@@ -61,6 +63,80 @@ class po_stats(Command):
                    (po, trans / all_, fuzzy / all_))
 
 
+class update_po(Command):
+
+    description = "update po files"
+    user_options = [
+        ("lang=", None, "force update <lang>.po"),
+    ]
+
+    def initialize_options(self):
+        self.po_directory = None
+        self.lang = None
+
+    def finalize_options(self):
+        self.po_directory = self.distribution.po_directory
+        self.po_package = self.distribution.po_package
+        self.po_files = glob.glob(os.path.join(self.po_directory, "*.po"))
+        self.pot_file = os.path.join(
+            self.po_directory, self.po_package + ".pot")
+
+    def _update_pot(self):
+        gettextutil.update_pot(self.po_directory, self.po_package)
+
+    def _update_po(self, po):
+        assert po in self.po_files
+        lang_code = os.path.basename(po[:-3])
+        gettextutil.update_po(self.po_directory, self.po_package, lang_code)
+
+    def run(self):
+        try:
+            gettextutil.check_version()
+        except gettextutil.GettextError as e:
+            raise SystemExit(e)
+
+        # if lang is given, force update pot and the specific po
+        if self.lang is not None:
+            po = os.path.join(self.po_directory, self.lang + ".po")
+            if po not in self.po_files:
+                raise SystemExit("Error: %r not found" % po)
+            self._update_pot()
+            self._update_po(po)
+            return
+
+        infilename = os.path.join(self.po_directory, "POTFILES.in")
+        with open(infilename, "rb") as h:
+            infiles = h.read().splitlines()
+
+        # if any of the in files is newer than the pot, update the pot
+        for filename in infiles:
+            if newer(filename, self.pot_file):
+                self._update_pot()
+                break
+        else:
+            print "not pot update"
+
+        # if the pot file is newer than any of the po files, update that po
+        for po in self.po_files:
+            if newer(self.pot_file, po):
+                self._update_po(po)
+
+
+def strip_pot_date(path):
+    """strip POT-Creation-Date from po/pot"""
+
+    done = False
+    lines = []
+    for line in open(path, "rb"):
+        if not done and line.startswith('"POT-Creation-Date:'):
+            done = True
+            continue
+        lines.append(line)
+
+    with open(path, "wb") as h:
+        h.write("".join(lines))
+
+
 class build_mo(Command):
     """build message catalog files
 
@@ -70,52 +146,51 @@ class build_mo(Command):
 
     description = "build message catalog files"
     user_options = []
-    build_base = None
-    po_package = None
-    po_files = None
-    pot_file = None
 
     def initialize_options(self):
-        pass
+        self.skip_po_update = None
+        self.build_base = None
+        self.po_package = None
+        self.po_files = None
+        self.pot_file = None
 
     def finalize_options(self):
         self.po_directory = self.distribution.po_directory
-        self.shortcuts = self.distribution.shortcuts
         self.po_package = self.distribution.po_package
         self.set_undefined_options('build', ('build_base', 'build_base'))
+        self.set_undefined_options(
+            'build', ('skip_po_update', 'skip_po_update'))
         self.po_files = glob.glob(os.path.join(self.po_directory, "*.po"))
         self.pot_file = os.path.join(
             self.po_directory, self.po_package + ".pot")
 
     def run(self):
-        if find_executable("intltool-update") is None:
-            raise SystemExit("Error: 'intltool' not found.")
         if find_executable("msgfmt") is None:
             raise SystemExit("Error: 'gettext' not found.")
-        basepath = os.path.join(self.build_base, 'share', 'locale')
-        infilename = os.path.join(self.po_directory, "POTFILES.in")
-        infiles = file(infilename).read().splitlines()
-        pot_name = os.path.join(
-            self.po_directory, self.po_package + ".pot")
 
-        for filename in infiles:
-            if newer(filename, pot_name):
-                oldpath = os.getcwd()
-                os.chdir(self.po_directory)
-                self.spawn(["intltool-update", "--pot",
-                            "--gettext-package", self.po_package])
-                for po in self.po_files:
-                    self.spawn(["intltool-update", "--dist",
-                                "--gettext-package", self.po_package,
-                                os.path.basename(po[:-3])])
-                os.chdir(oldpath)
+        # It's OK to skip po update for building release tarballs, since
+        # things are updated right before release...
+        if not self.skip_po_update:
+            self.run_command("update_po")
+
+        basepath = os.path.join(self.build_base, 'share', 'locale')
+
         for po in self.po_files:
             language = os.path.basename(po).split(".")[0]
             fullpath = os.path.join(basepath, language, "LC_MESSAGES")
             destpath = os.path.join(fullpath, self.po_package + ".mo")
             if newer(po, destpath):
                 self.mkpath(fullpath)
-                self.spawn(["msgfmt", "-o", destpath, po])
+
+                # strip POT-Creation-Date from po/mo to make build reproducible
+                fd, temp_path = mkstemp(".po")
+                try:
+                    os.close(fd)
+                    shutil.copy(po, temp_path)
+                    strip_pot_date(temp_path)
+                    self.spawn(["msgfmt", "-o", destpath, temp_path])
+                finally:
+                    os.remove(temp_path)
 
 
 class install_mo(Command):
@@ -128,20 +203,17 @@ class install_mo(Command):
     description = "install message catalog files"
     user_options = []
 
-    skip_build = None
-    build_base = None
-    install_base = None
-    root = None
-
     def initialize_options(self):
+        self.skip_build = None
+        self.build_base = None
+        self.install_dir = None
         self.outfiles = []
 
     def finalize_options(self):
         self.set_undefined_options('build', ('build_base', 'build_base'))
         self.set_undefined_options(
             'install',
-            ('root', 'root'),
-            ('install_base', 'install_base'),
+            ('install_data', 'install_dir'),
             ('skip_build', 'skip_build'))
 
     def get_outputs(self):
@@ -151,9 +223,7 @@ class install_mo(Command):
         if not self.skip_build:
             self.run_command('build_mo')
         src = os.path.join(self.build_base, "share", "locale")
-        dest = os.path.join(self.install_base, "share", "locale")
-        if self.root is not None:
-            dest = change_root(self.root, dest)
+        dest = os.path.join(self.install_dir, "share", "locale")
         out = self.copy_tree(src, dest)
         self.outfiles.extend(out)
 
