@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
 #
 # This software and accompanying documentation, if any, may be freely
@@ -7,15 +8,13 @@
 
 import os
 
-from distutils.util import change_root
 from distutils.core import Command
 
 
 class install_search_provider(Command):
 
     user_options = []
-    root = None
-    prefix = None
+    install_dir = None
     search_provider = None
 
     def initialize_options(self):
@@ -23,8 +22,7 @@ class install_search_provider(Command):
 
     def finalize_options(self):
         self.set_undefined_options('install',
-                                   ('root', 'root'),
-                                   ('install_base', 'prefix'))
+                                   ('install_data', 'install_dir'))
 
         self.search_provider = self.distribution.search_provider
 
@@ -33,10 +31,7 @@ class install_search_provider(Command):
 
     def run(self):
         basepath = os.path.join(
-            self.prefix, 'share', 'gnome-shell', 'search-providers')
-        if self.root is not None:
-            basepath = change_root(self.root, basepath)
-
+            self.install_dir, 'share', 'gnome-shell', 'search-providers')
         out = self.mkpath(basepath)
         self.outfiles.extend(out or [])
         (out, _) = self.copy_file(self.search_provider, basepath)
