@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2007 Joe Wreschnig
 #
 # This software and accompanying documentation, if any, may be freely
@@ -8,7 +9,6 @@
 import os
 
 from distutils.dep_util import newer
-from distutils.util import change_root
 from distutils.core import Command
 
 
@@ -61,11 +61,10 @@ class install_shortcuts(Command):
     description = "install .desktop files"
     user_options = []
 
-    prefix = None
+    install_dir = None
     skip_build = None
     shortcuts = None
     build_base = None
-    root = None
 
     def initialize_options(self):
         self.outfiles = []
@@ -74,8 +73,7 @@ class install_shortcuts(Command):
         self.set_undefined_options('build', ('build_base', 'build_base'))
         self.set_undefined_options(
             'install',
-            ('root', 'root'),
-            ('install_base', 'prefix'),
+            ('install_data', 'install_dir'),
             ('skip_build', 'skip_build'))
 
         self.set_undefined_options(
@@ -87,9 +85,7 @@ class install_shortcuts(Command):
     def run(self):
         if not self.skip_build:
             self.run_command('build_shortcuts')
-        basepath = os.path.join(self.prefix, 'share', 'applications')
-        if self.root is not None:
-            basepath = change_root(self.root, basepath)
+        basepath = os.path.join(self.install_dir, 'share', 'applications')
         srcpath = os.path.join(self.build_base, 'share', 'applications')
         out = self.mkpath(basepath)
         self.outfiles.extend(out or [])
