@@ -11,32 +11,30 @@
 # published by the Free Software Foundation.
 
 import sys
+
 import os
-from quodlibet.cli import process_arguments, is_running, control
+
+from quodlibet.cli import process_arguments, exit_
 from quodlibet.util.dprint import print_d, print_
-
-if sys.version_info[0] != 2:
-    try:
-        os.execvp("python2", ["python"] + sys.argv)
-    except OSError:
-        pass
-
-import sys
+from quodlibet.util import set_win32_unicode_argv
 
 
 def main():
-    startup_actions = process_arguments()
-
-    from quodlibet import const
-    if is_running() and not const.DEBUG:
-        print_(_("Quod Libet is already running."))
-        control('focus')
+    try:
+        # we want basic commands not to import gtk (doubles process time)
+        assert "gi.repository.Gtk" not in sys.modules
+        sys.modules["gi.repository.Gtk"] = None
+        startup_actions, cmds_todo = process_arguments()
+    finally:
+        sys.modules.pop("gi.repository.Gtk", None)
 
     import quodlibet
     from quodlibet import app
-    quodlibet._init_signal(app.quit)
+    from quodlibet.qltk import add_signal_watch, icons
+    add_signal_watch(app.quit)
 
     import quodlibet.player
+    import quodlibet.library
     from quodlibet import config
     from quodlibet import browsers
     from quodlibet import const
@@ -44,15 +42,22 @@ def main():
 
     config.init(const.CONFIG)
 
-    library = quodlibet.init(library=const.LIBRARY,
-                             icon="quodlibet",
-                             name="Quod Libet",
-                             title=const.PROCESS_TITLE_QL)
+    app.name = "Quod Libet"
+    app.id = "quodlibet"
+
+    quodlibet.init(icon=icons.QUODLIBET, name=app.name, proc_title=app.id)
+
+    print_d("Initializing main library (%s)" % (
+            quodlibet.util.path.unexpand(const.LIBRARY)))
+
+    library = quodlibet.library.init(const.LIBRARY)
     app.library = library
 
     from quodlibet.player import PlayerError
+    backend = os.environ.get(
+        "QUODLIBET_BACKEND", config.get("player", "backend"))
     # this assumes that nullbe will always succeed
-    for backend in [config.get("player", "backend"), "nullbe"]:
+    for backend in [backend, "nullbe"]:
         try:
             player = quodlibet.init_backend(backend, app.librarian)
         except PlayerError as error:
@@ -102,34 +107,58 @@ def main():
 
     from quodlibet.qltk.songsmenu import SongsMenu
     SongsMenu.init_plugins()
-    from quodlibet.util.cover.manager import cover_plugins
-    cover_plugins.init_plugins()
+
+    from quodlibet.util.cover import CoverManager
+    app.cover_manager = CoverManager()
+    app.cover_manager.init_plugins()
+
     from quodlibet.plugins.playlist import PLAYLIST_HANDLER
     PLAYLIST_HANDLER.init_plugins()
 
+    from gi.repository import GLib
+
+    def exec_commands(*args):
+        for cmd in cmds_todo:
+            try:
+                resp = cmd_registry.run(app, *cmd)
+            except CommandError:
+                pass
+            else:
+                if resp is not None:
+                    print_(resp, end="")
+
     from quodlibet.qltk.quodlibetwindow import QuodLibetWindow
-    app.window = window = QuodLibetWindow(library, player)
+    # Call exec_commands after the window is restored, but make sure
+    # it's after the mainloop has started so everything is set up.
+    app.window = window = QuodLibetWindow(
+        library, player,
+        restore_cb=lambda:
+            GLib.idle_add(exec_commands, priority=GLib.PRIORITY_HIGH))
 
     from quodlibet.plugins.events import EventPluginHandler
     pm.register_handler(EventPluginHandler(library.librarian, player))
 
-    from quodlibet.qltk import mmkeys_ as mmkeys
-    from quodlibet.qltk.remote import FSInterface, FIFOControl
-    from quodlibet.qltk.tracker import SongTracker
+    from quodlibet.mmkeys import MMKeysHandler
+    from quodlibet.remote import Remote, RemoteError
+    from quodlibet.commands import registry as cmd_registry, CommandError
+    from quodlibet.qltk.tracker import SongTracker, FSInterface
     try:
         from quodlibet.qltk.dbus_ import DBusHandler
     except ImportError:
         DBusHandler = lambda player, library: None
 
-    mmkeys.init(window, player)
+    mmkeys_handler = MMKeysHandler(app.name, window, player)
+    if "QUODLIBET_NO_MMKEYS" not in os.environ:
+        mmkeys_handler.start()
     fsiface = FSInterface(player)
-    quodlibet.quit_add(1, fsiface.destroy)
-
-    fifoctrl = FIFOControl(app)
-    quodlibet.quit_add(1, fifoctrl.destroy)
+    remote = Remote(app, cmd_registry)
+    try:
+        remote.start()
+    except RemoteError:
+        exit_(1, True)
 
     DBusHandler(player, library)
-    SongTracker(library.librarian, player, window.playlist)
+    tracker = SongTracker(library.librarian, player, window.playlist)
 
     from quodlibet.qltk import session
     session.init("quodlibet")
@@ -141,14 +170,28 @@ def main():
 
     # restore browser windows
     from quodlibet.qltk.browser import LibraryBrowser
-    from gi.repository import GLib
-    GLib.idle_add(LibraryBrowser.restore, library, priority=GLib.PRIORITY_HIGH)
+    GLib.idle_add(LibraryBrowser.restore, library, player,
+                  priority=GLib.PRIORITY_HIGH)
 
-    quodlibet.main(window)
+    def before_quit():
+        print_d("Saving active browser state")
+        try:
+            app.browser.save()
+        except NotImplementedError:
+            pass
 
-    print_d("Shutting down player device %r." % player.version_info)
-    player.destroy()
-    quodlibet.library.save(force=True)
+        print_d("Shutting down player device %r." % player.version_info)
+        player.destroy()
+
+    quodlibet.main(window, before_quit=before_quit)
+
+    quodlibet.finish_first_session(app.id)
+    mmkeys_handler.quit()
+    remote.stop()
+    fsiface.destroy()
+
+    tracker.destroy()
+    quodlibet.library.save()
 
     config.save(const.CONFIG)
 
@@ -156,4 +199,5 @@ def main():
 
 
 if __name__ == "__main__":
+    set_win32_unicode_argv()
     main()
