@@ -9,7 +9,7 @@
 import os
 
 from quodlibet import const
-from quodlibet.util.modulescanner import load_dir_modules
+from quodlibet.util.importhelper import load_dir_modules
 from quodlibet.browsers._base import Browser
 
 
@@ -19,6 +19,10 @@ browsers = []
 def init():
     global browsers
 
+    # ignore double init (for the test suite)
+    if browsers:
+        return
+
     this_dir = os.path.dirname(__file__)
     load_pyc = os.name == 'nt'
     modules = load_dir_modules(this_dir,
