@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2004-2008 Joe Wreschnig
 #           2009-2013 Nick Boultbee
 #           2011-2014 Christoph Reiter
@@ -6,9 +7,6 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from StringIO import StringIO
-import csv
-import os
 import shutil
 
 import const
@@ -112,6 +110,10 @@ INITIAL = {
 
         # search as soon as text is typed into search box
         "eager_search": "true",
+
+        # tags which get searched in addition to the ones present in the
+        # song list, separate with ","
+        "search_tags": "",
     },
     "rename": {
         "spaces": "false",
@@ -138,7 +140,8 @@ INITIAL = {
         "save_email": const.EMAIL,
         "alltags": "true", # show all tags, or just "human-readable" ones
         # Skip dialog to save or revert changes
-        "auto_save_changes": "false"
+        "auto_save_changes": "false",
+        "default_tags": "", # e.g. "title,artist"
     },
     "albumart": {
         "round": "true", # use rounded corners for artwork thumbnails
@@ -159,6 +162,8 @@ getint = _config.getint
 getfloat = _config.getfloat
 getstringlist = _config.getstringlist
 setstringlist = _config.setstringlist
+getlist = _config.getlist
+setlist = _config.setlist
 set = _config.set
 setdefault = _config.setdefault
 write = _config.write
