@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2014 Christoph Reiter <reiter.christoph@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -240,7 +241,10 @@ class MPDService(object):
         self._idle_subscriptions[connection] = subsystems
 
     def unregister_idle(self, connection):
-        del self._idle_subscriptions[connection]
+        try:
+            del self._idle_subscriptions[connection]
+        except KeyError:
+            pass
 
     def emit_changed(self, subsystem):
         for conn, subs in self._idle_subscriptions.iteritems():
@@ -819,7 +823,7 @@ def _cmd_outputs(conn, service, args):
 @MPDConnection.Command("commands")
 def _cmd_commands(conn, service, args):
     for name in conn.list_commands():
-        conn.write_line(unicode(name))
+        conn.write_line(u"command: " + unicode(name))
 
 
 @MPDConnection.Command("tagtypes")
