@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2012 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -15,9 +16,8 @@ from quodlibet.plugins.songsmenu import SongsMenuPlugin
 class FilterBrowser(SongsMenuPlugin):
     PLUGIN_ID = 'filterbrowser'
     PLUGIN_NAME = _('Filter on Directory')
-    PLUGIN_DESC = _("Filter on directory in a new browser window.")
+    PLUGIN_DESC = _("Filters on directory in a new browser window.")
     PLUGIN_ICON = Gtk.STOCK_INDEX
-    PLUGIN_VERSION = '0.1'
 
     def plugin_songs(self, songs):
         tag = "~dirname"
@@ -26,5 +26,6 @@ class FilterBrowser(SongsMenuPlugin):
         for song in songs:
             values.extend(song.list(tag))
 
-        browser = LibraryBrowser.open(browsers.get("SearchBar"), app.library)
+        browser = LibraryBrowser.open(
+            browsers.get("SearchBar"), app.library, app.player)
         browser.browser.filter(tag, set(values))
