@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
 #           2013 Nick Boultbee
 #           2013,2014 Christoph Reiter
@@ -12,16 +13,17 @@ These classes are the most basic library classes. As such they are the
 least useful but most content-agnostic.
 """
 
+from pickle import Unpickler
+from cStringIO import StringIO
 import cPickle as pickle
 import os
 import shutil
-import threading
 import time
 
 from gi.repository import GObject
 
 from quodlibet.formats import MusicFile
-from quodlibet.parse import Query
+from quodlibet.query import Query
 from quodlibet.qltk.notif import Task
 from quodlibet.util.collection import Album
 from quodlibet.util.collections import DictMixin
@@ -29,7 +31,8 @@ from quodlibet import util
 from quodlibet import const
 from quodlibet import formats
 from quodlibet.util.dprint import print_d, print_w
-from quodlibet.util.path import fsdecode, expanduser, unexpand, mkdir
+from quodlibet.util.path import fsdecode, expanduser, unexpand, mkdir, \
+    normalize_path
 
 
 class Library(GObject.GObject, DictMixin):
@@ -217,6 +220,34 @@ def dump_items(filename, items):
         pickle.dump(items, fileobj, 1)
 
 
+def unpickle_save(data, default, type_=dict):
+    """Unpickle a list of `type_` subclasses and skip items for which the
+    class is missing.
+
+    In case not just the class lookup fails, returns default.
+    """
+
+    class dummy(type_):
+        pass
+
+    class SaveUnpickler(Unpickler):
+
+        def find_class(self, module, name):
+            try:
+                return Unpickler.find_class(self, module, name)
+            except (ImportError, AttributeError):
+                return dummy
+
+    fileobj = StringIO(data)
+
+    try:
+        items = SaveUnpickler(fileobj).load()
+    except Exception:
+        return default
+
+    return [i for i in items if not isinstance(i, dummy)]
+
+
 def load_items(filename, default=None):
     """Load items from disk.
 
@@ -254,7 +285,10 @@ def load_items(filename, default=None):
         except EnvironmentError:
             util.print_exc()
 
-        items = default
+        # try to skip items for which the class is missing
+        # XXX: we assume the items are dict subclasses here.. while nothing
+        # else does
+        items = unpickle_save(data, default)
 
     return items
 
@@ -264,9 +298,6 @@ class PicklingMixin(object):
 
     filename = None
 
-    def __init__(self):
-        self._save_lock = threading.Lock()
-
     def load(self, filename):
         """Load a library from a file, containing a picked list.
 
@@ -290,15 +321,14 @@ class PicklingMixin(object):
         if filename is None:
             filename = self.filename
 
-        with self._save_lock:
-            print_d("Saving contents to %r." % filename, self)
+        print_d("Saving contents to %r." % filename, self)
 
-            try:
-                dump_items(filename, self.get_content())
-            except EnvironmentError:
-                print_w("Couldn't save library to path: %r" % filename)
-            else:
-                self.dirty = False
+        try:
+            dump_items(filename, self.get_content())
+        except EnvironmentError:
+            print_w("Couldn't save library to path: %r" % filename)
+        else:
+            self.dirty = False
 
 
 class PicklingLibrary(Library, PicklingMixin):
@@ -806,13 +836,14 @@ class SongFileLibrary(SongLibrary, FileLibrary):
         Otherwise, None is returned.
         """
 
+        key = normalize_path(filename, True)
         song = None
-        if filename not in self._contents:
+        if key not in self._contents:
             song = MusicFile(filename)
             if song and add:
                 self.add([song])
         else:
             print_d("Already got file %r." % filename)
-            song = self._contents[filename]
+            song = self._contents[key]
 
         return song
