@@ -12,10 +12,12 @@ from gi.repository import Gtk, GObject, GLib
 from quodlibet import config
 from quodlibet import const
 
-from quodlibet.parse import Query
+from quodlibet.query import Query
 from quodlibet.qltk.cbes import ComboBoxEntrySave
-from quodlibet.qltk.x import Button
-from quodlibet.util import limit_songs
+from quodlibet.qltk.entry import QueryValidator
+from quodlibet.qltk.ccb import ConfigCheckMenuItem
+from quodlibet.qltk.x import SeparatorMenuItem
+from quodlibet.util import limit_songs, DeferredSignal
 
 
 class SearchBarBox(Gtk.HBox):
@@ -43,18 +45,18 @@ class SearchBarBox(Gtk.HBox):
             filename = os.path.join(const.USERDIR, "lists", "queries")
 
         combo = ComboBoxEntrySave(filename, count=8,
-                validator=Query.is_valid_color, title=_("Saved Searches"),
-                edit_title=_("Edit saved searches..."))
+                validator=QueryValidator, title=_("Saved Searches"),
+                edit_title=_(u"Edit saved searches…"))
+
+        self.__deferred_changed = DeferredSignal(
+            self.__filter_changed, timeout=self.timeout, owner=self)
 
-        self.__refill_id = None
         self.__combo = combo
         entry = combo.get_child()
         self.__entry = entry
         if completion:
             entry.set_completion(completion)
 
-        self.connect('destroy', lambda w: w.__remove_timeout())
-
         self.__sig = combo.connect('text-changed', self.__text_changed)
 
         entry.connect('clear', self.__filter_changed)
@@ -76,17 +78,13 @@ class SearchBarBox(Gtk.HBox):
             accel_group.connect(key, mod, 0,
                     lambda *x: entry.mnemonic_activate(True))
 
-        self.show_all()
-
-    def __inhibit(self):
-        self.__combo.handler_block(self.__sig)
-
-    def __uninhibit(self):
-        self.__combo.handler_unblock(self.__sig)
+        for child in self.get_children():
+            child.show_all()
 
     def set_text(self, text):
-        # remove the timeout
-        self.__remove_timeout()
+        """Set the text without firing any signals"""
+
+        self.__deferred_changed.abort()
 
         # deactivate all signals and change the entry text
         self.__inhibit()
@@ -94,19 +92,35 @@ class SearchBarBox(Gtk.HBox):
         self.__uninhibit()
 
     def get_text(self):
+        """Get the active text as unicode"""
+
         return self.__entry.get_text().decode("utf-8")
 
     def changed(self):
         """Triggers a filter-changed signal if the current text
-        is a parsable query"""
+        is a parsable query
+        """
+
         self.__filter_changed()
 
-    def __menu(self, entry, menu):
-        self.show_menu(menu)
+    def __inhibit(self):
+        self.__combo.handler_block(self.__sig)
 
-    def show_menu(self, menu):
-        """Overwrite this method for providing a menu"""
-        pass
+    def __uninhibit(self):
+        self.__combo.handler_unblock(self.__sig)
+
+    def __menu(self, entry, menu):
+        sep = SeparatorMenuItem()
+        sep.show()
+        menu.prepend(sep)
+
+        cb = ConfigCheckMenuItem(
+            _("Search after _typing"), 'settings', 'eager_search',
+            populate=True)
+        cb.set_tooltip_text(
+            _("Show search results after the user stops typing."))
+        cb.show()
+        menu.prepend(cb)
 
     def __mnemonic_activate(self, label, group_cycling):
         widget = label.get_mnemonic_widget()
@@ -119,7 +133,7 @@ class SearchBarBox(Gtk.HBox):
         if args and not config.getboolean('settings', 'eager_search'):
             return
 
-        text = entry.get_text().decode('utf-8').strip()
+        text = self.get_text().strip()
         if text and Query.is_parsable(text):
             # Adding the active text to the model triggers a changed signal
             # (get_active is no longer -1), so inhibit
@@ -128,18 +142,11 @@ class SearchBarBox(Gtk.HBox):
             self.__combo.write()
             self.__uninhibit()
 
-    def __remove_timeout(self):
-        if self.__refill_id is not None:
-            GLib.source_remove(self.__refill_id)
-            self.__refill_id = None
-
     def __filter_changed(self, *args):
-        self.__remove_timeout()
-
-        text = self.__entry.get_text().decode('utf-8')
+        self.__deferred_changed.abort()
+        text = self.get_text()
         if Query.is_parsable(text):
-            self.__refill_id = GLib.idle_add(
-                self.emit, 'query-changed', text)
+            GLib.idle_add(self.emit, 'query-changed', text)
 
     def __text_changed(self, *args):
         # the combobox has an active entry selected -> no timeout
@@ -152,14 +159,7 @@ class SearchBarBox(Gtk.HBox):
         if not config.getboolean('settings', 'eager_search'):
             return
 
-        # remove the timeout
-        self.__remove_timeout()
-
-        # parse and new timeout
-        text = self.__entry.get_text().decode('utf-8')
-        if Query.is_parsable(text):
-            self.__refill_id = GLib.timeout_add(
-                    self.timeout, self.__filter_changed)
+        self.__deferred_changed()
 
 
 class LimitSearchBarBox(SearchBarBox):
@@ -185,10 +185,13 @@ class LimitSearchBarBox(SearchBarBox):
             label.set_use_underline(True)
             self.pack_start(limit, True, True, 0)
 
-            self.__weight = Gtk.CheckButton(_("_Weight"), use_underline=True)
+            self.__weight = Gtk.CheckButton(
+                label=_("_Weight"), use_underline=True)
             self.__weight.connect("toggled", self.__changed)
             self.pack_start(self.__weight, True, True, 0)
-            map(lambda w: w.show(), self.get_children())
+
+            for child in self.get_children():
+                child.show()
 
         def __changed(self, *args):
             self.emit("changed")
