@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig, Michael Urman
 #
 # This program is free software; you can redistribute it and/or modify
@@ -92,7 +93,7 @@ class WaitLoadBase(object):
         if self.count:
             t = (time.time() - self._start_time) / self.current
             remaining = math.ceil((self.count - self.current) * t)
-            values.setdefault("remaining", util.format_time(remaining))
+            values.setdefault("remaining", util.format_time_display(remaining))
         self._label.set_markup(self._text % values)
 
         while not self.quit and (self.paused or Gtk.events_pending()):
@@ -214,5 +215,8 @@ class WaitLoadBar(WaitLoadBase, Gtk.HBox):
 
     def step(self, **values):
         ret = super(WaitLoadBar, self).step(**values)
-        self._progress.set_text(_("%d of %d") % (self.current, self.count))
+        self._progress.set_text(_("%(current)d of %(all)d") % {
+            "current": self.current,
+            "all": self.count,
+        })
         return ret
