@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2013 Joe Wreschnig, Michael Urman, Iñigo Serna,
-#                     Christoph Reiter, Steven Robertson, Nick Boultbee
+#                     Christoph Reiter, Steven Robertson
+#           2011-2014 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,21 +9,24 @@
 
 from __future__ import absolute_import
 
-from gi.repository import GLib
-
 import os
 import random
 
 from quodlibet import util
 from quodlibet import config
-from quodlibet.formats._audio import PEOPLE, TAG_TO_SORT, INTERN_NUM_DEFAULT
-from quodlibet.util import thumbnails
+from quodlibet.formats._audio import TAG_TO_SORT, INTERN_NUM_DEFAULT
+from quodlibet.formats._audio import PEOPLE as _PEOPLE
 from collections import Iterable
 from quodlibet.util.path import escape_filename, unescape_filename
 from quodlibet.util.path import bytes2fsnative, is_fsnative, fsnative2bytes
 from .collections import HashedList
 
 
+PEOPLE = list(_PEOPLE)
+# Collections value albumartist more than song artist (Issue 1034)
+PEOPLE.remove("albumartist")
+PEOPLE.insert(0, "albumartist")
+
 ELPOEP = list(reversed(PEOPLE))
 PEOPLE_SCORE = [100 ** i for i in xrange(len(PEOPLE))]
 
@@ -267,7 +271,7 @@ class Album(Collection):
     """Like a `Collection` but adds cover scanning, some attributes for sorting
     and uses a set for the songs."""
 
-    COVER_SIZE = 50
+    COVER_SIZE = 48
 
     cover = None
     scanned = False
@@ -280,8 +284,13 @@ class Album(Collection):
     def genre(self):
         return util.human_sort_key(self.get("genre").split("\n")[0])
 
-    date = property(lambda self: self.get("date"))
-    title = property(lambda self: self.get("album"))
+    @property
+    def date(self):
+        return self.get("date")
+
+    @property
+    def title(self):
+        return self.get("album")
 
     def __init__(self, song):
         super(Album, self).__init__()
@@ -300,24 +309,23 @@ class Album(Collection):
         self.__dict__.pop("peoplesort", None)
         self.__dict__.pop("genre", None)
 
-    def scan_cover(self, force=False, scale_factor=1):
+    def scan_cover(self, force=False, scale_factor=1,
+            callback=None, cancel=None):
         if (self.scanned and not force) or not self.songs:
             return
         self.scanned = True
 
-        song = iter(self.songs).next()
-        cover = song.find_cover()
+        def set_cover_cb(pixbuf):
+            self.cover = pixbuf
+            callback()
 
-        if cover is not None:
-            s = self.COVER_SIZE * scale_factor - scale_factor * 2
-
-            try:
-                round = config.getboolean("albumart", "round")
-                self.cover = thumbnails.get_thumbnail(cover.name, (s, s))
-                self.cover = thumbnails.add_border(
-                    self.cover, 30, round=round, width=scale_factor)
-            except GLib.GError:
-                return
+        from quodlibet import app
+        s = self.COVER_SIZE * scale_factor
+        if callback is not None:
+            app.cover_manager.get_pixbuf_many_async(
+                self.songs, s, s, cancel, set_cover_cb)
+        else:
+            self.cover = app.cover_manager.get_pixbuf_many(self.songs, s, s)
 
     def __repr__(self):
         return "Album(%s)" % repr(self.key)
