@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -5,7 +6,9 @@
 # published by the Free Software Foundation
 
 from quodlibet.formats._audio import AudioFile
-from . import PluginTestCase, modules, skipUnless
+
+from tests import skipUnless
+from . import PluginTestCase, modules
 
 brainz = modules.get("MusicBrainz lookup", None)
 
