@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter, Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
@@ -11,13 +12,26 @@ from tests.plugin import PluginTestCase
 from quodlibet import config
 from quodlibet.qltk.songsmenu import SongsMenuPluginHandler
 from quodlibet.formats._audio import AudioFile
+from quodlibet.util.path import fsnative
 from quodlibet.library import SongLibrary, SongLibrarian
 
 SONGS = [
-    AudioFile({"title": "one", "artist": "piman", "~filename": "/dev/null"}),
-    AudioFile({"title": "two", "artist": "mu", "~filename": "/dev/zero"}),
-    AudioFile({"title": "three", "artist": "boris", "~filename": "/bin/ls"})
-    ]
+    AudioFile({
+        "title": "one",
+        "artist": "piman",
+        "~filename": fsnative(u"/dev/null"),
+    }),
+    AudioFile({
+        "title": "two",
+        "artist": "mu",
+        "~filename": fsnative(u"/dev/zero"),
+    }),
+    AudioFile({
+        "title": "three",
+        "artist": "boris",
+        "~filename": fsnative(u"/bin/ls"),
+    }),
+]
 SONGS.sort()
 
 for song in SONGS:
@@ -27,7 +41,7 @@ for song in SONGS:
 class TPluginsSongsMenu(PluginTestCase):
     def setUp(self):
         config.init()
-        self.h = SongsMenuPluginHandler(lambda msg: True)
+        self.h = SongsMenuPluginHandler()
         library = SongLibrary()
         library.librarian = SongLibrarian()
         self.lib = library
