@@ -1,11 +1,9 @@
+# -*- coding: utf-8 -*-
 from tests import TestCase
 
-from gi.repository import Gtk
-
 from quodlibet.library import SongLibrary
 from quodlibet.formats._audio import AudioFile
 from quodlibet.browsers.iradio import InternetRadio, IRFile, QuestionBar
-from quodlibet.player.nullbe import NullPlayer
 import quodlibet.config
 
 quodlibet.config.RATINGS = quodlibet.config.HardCodedRatingsPrefs()
@@ -21,11 +19,11 @@ class TQuestionBar(TestCase):
 class TInternetRadio(TestCase):
     def setUp(self):
         quodlibet.config.init()
-        self.bar = InternetRadio(SongLibrary(), NullPlayer())
+        self.bar = InternetRadio(SongLibrary())
 
     def test_can_filter(self):
-        for key in ["foo", "title", "fake~key", "~woobar", "~#huh"]:
-            self.failIf(self.bar.can_filter(key))
+        self.assertTrue(self.bar.can_filter("foo"))
+        self.assertTrue(self.bar.can_filter_text())
 
     def tearDown(self):
         self.bar.destroy()
