@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,16 +19,24 @@ class Tconst(TestCase):
     def test_branch_name(self):
         devnull = open(os.devnull, 'w')
         try:
-            subprocess.check_call(["hg", "status"], stdout=devnull)
+            subprocess.check_call(["git", "status"], stdout=devnull)
         except (OSError, subprocess.CalledProcessError):
             # no active hg repo, skip
             return
 
-        p = subprocess.Popen(["hg", "id", "-b"], stdout=subprocess.PIPE)
+        p = subprocess.Popen(
+            ["git", "rev-parse", "--abbrev-ref", "HEAD"],
+            stdout=subprocess.PIPE)
         branch = p.communicate()[0].strip()
         self.failIf(p.returncode)
 
-        self.failUnlessEqual(branch, const.BRANCH_NAME)
+        # only check for stable/dev branches, no feature branches
+        if branch == "master" or branch.startswith("quodlibet"):
+            self.failUnlessEqual(branch, const.BRANCH_NAME)
+
+    def test_authors(self):
+        # Noting that <= is subset operator on sets...
+        self.assertLessEqual(set(const.MAIN_AUTHORS), set(const.AUTHORS))
 
     def test_path_types(self):
         self.assertTrue(is_fsnative(const.USERDIR))
