@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2012 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -8,6 +9,7 @@ from tests import TestCase
 
 from quodlibet.qltk.bookmarks import EditBookmarks, MenuItems
 from quodlibet.player.nullbe import NullPlayer
+from quodlibet.util.path import fsnative
 from quodlibet.library import SongLibrary
 from quodlibet.formats._audio import AudioFile
 from quodlibet import config
@@ -19,7 +21,7 @@ class TBookmarks(TestCase):
         player = NullPlayer()
         song = AudioFile()
         song.bookmarks = [(10, "bla")]
-        song.sanitize("/")
+        song.sanitize(fsnative(u"/"))
         player.song = song
         self.player = player
 
