@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -89,11 +90,33 @@ class TObjectStore(_TObjectStore):
         ObjectStore(object)
         self.failUnlessRaises(ValueError, ObjectStore, object, object)
 
+    def test_iter_path_changed(self):
+        m = ObjectStore()
+
+        def handler(model, path, iter_, result):
+            result[0] += 1
+
+        result = [0]
+        m.connect("row-changed", handler, result)
+        m.append([object()])
+
+        iter_ = m.get_iter_first()
+        m.iter_changed(iter_)
+        self.assertEqual(result[0], 1)
+        m.path_changed(m.get_path(iter_))
+        self.assertEqual(result[0], 2)
+
     def test_append_many(self):
         m = ObjectStore()
         m.append_many(range(10))
         self.failUnlessEqual([r[0] for r in m], range(10))
 
+    def test_append_many_set(self):
+        m = ObjectStore()
+        m.append_many(set())
+        m.append_many(set(range(10)))
+        self.failUnlessEqual(set([r[0] for r in m]), set(range(10)))
+
     def test_iter_append_many(self):
         m = ObjectStore()
         iters = list(m.iter_append_many(range(10)))
@@ -289,6 +312,22 @@ class TObjectTreeStore(_TObjectTreeStore):
         ObjectTreeStore(object)
         self.failUnlessRaises(ValueError, ObjectTreeStore, object, object)
 
+    def test_iter_path_changed(self):
+        m = ObjectTreeStore()
+
+        def handler(model, path, iter_, result):
+            result[0] += 1
+
+        result = [0]
+        m.connect("row-changed", handler, result)
+        m.append(None, [object()])
+
+        iter_ = m.get_iter_first()
+        m.iter_changed(iter_)
+        self.assertEqual(result[0], 1)
+        m.path_changed(m.get_path(iter_))
+        self.assertEqual(result[0], 2)
+
     def test_itervalues(self):
         m = ObjectTreeStore()
         obj = object()
