@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from tests import TestCase, AbstractTestCase
 
 from quodlibet.qltk.msg import *
@@ -22,10 +23,6 @@ class TErrorMessage(_TMessage):
     Kind = ErrorMessage
 
 
-class TConfirmAction(_TMessage):
-    Kind = ConfirmAction
-
-
 class TCancelRevertSave(TestCase):
     def setUp(self):
         self.win = CancelRevertSave(None)
@@ -35,3 +32,15 @@ class TCancelRevertSave(TestCase):
 
     def tearDown(self):
         self.win.destroy()
+
+
+class TFileReplace(TestCase):
+
+    def setUp(self):
+        self.win = ConfirmFileReplace(None, "")
+
+    def test_ctr(self):
+        pass
+
+    def tearDown(self):
+        self.win.destroy()
