@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2012,2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -7,6 +8,7 @@
 from tests import TestCase
 from quodlibet import config
 from quodlibet.qltk.ratingsmenu import RatingsMenuItem
+from quodlibet.util.path import fsnative
 from quodlibet.library import SongLibrary, SongLibrarian
 from quodlibet.formats._audio import AudioFile
 
@@ -22,7 +24,7 @@ class TRatingsMenuItem(TestCase):
     def test_menuitem(self):
         library = SongLibrary()
         library.librarian = SongLibrarian()
-        a = AudioFile({"~filename": "/foo"})
+        a = AudioFile({"~filename": fsnative(u"/foo")})
         a.sanitize()
         x = RatingsMenuItem([a], library)
         x.set_rating(0, [a], library)
