@@ -1,15 +1,21 @@
+# -*- coding: utf-8 -*-
+import os
+
 from tests import TestCase
 
 from gi.repository import Gtk
 
+from quodlibet import config
+from quodlibet import const
 from quodlibet.formats._audio import AudioFile
 from quodlibet.player.nullbe import NullPlayer
-from quodlibet.qltk.tracker import SongTracker
+from quodlibet.qltk.tracker import SongTracker, FSInterface
 from quodlibet.library import SongLibrary
 
 
 class TSongTracker(TestCase):
     def setUp(self):
+        config.init()
         self.p = NullPlayer()
         self.w = SongLibrary()
         self.s1 = AudioFile(
@@ -25,6 +31,9 @@ class TSongTracker(TestCase):
         while Gtk.events_pending():
             Gtk.main_iteration()
 
+    def test_destroy(self):
+        self.cm.destroy()
+
     def test_play(self):
         import time
         # Allow at least 2 second to elapse to simulate playing
@@ -63,3 +72,37 @@ class TSongTracker(TestCase):
 
     def tearDown(self):
         self.w.destroy()
+        config.quit()
+
+
+class TFSInterface(TestCase):
+    def setUp(self):
+        self.p = NullPlayer()
+        self.fs = FSInterface(self.p)
+
+    def do(self):
+        while Gtk.events_pending():
+            Gtk.main_iteration()
+
+    def test_init(self):
+        self.do()
+        self.failIf(os.path.exists(const.CURRENT))
+
+    def test_start(self):
+        self.p.emit('song_started', AudioFile({"woo": "bar", "~#length": 10}))
+        self.do()
+        self.failUnless("woo=bar\n" in file(const.CURRENT).read())
+
+    def test_song_ended(self):
+        self.p.emit('song-started', AudioFile({"woo": "bar", "~#length": 10}))
+        self.do()
+        self.p.emit('song-ended', {}, False)
+        self.do()
+        self.failIf(os.path.exists(const.CURRENT))
+
+    def tearDown(self):
+        self.p.destroy()
+        try:
+            os.unlink(const.CURRENT)
+        except EnvironmentError:
+            pass
