@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from tests import TestCase
 
 from quodlibet.qltk.window import Window
@@ -6,6 +7,9 @@ from quodlibet.util import InstanceTracker
 
 class TWindows(TestCase):
 
+    def test_ctr(self):
+        Window().destroy()
+
     def test_instance_tracking(self):
 
         class SomeWindow(Window, InstanceTracker):
@@ -32,3 +36,12 @@ class TWindows(TestCase):
         Window.prevent_inital_show(False)
         w.show_maybe()
         self.assertTrue(w.get_visible())
+
+    def test_use_header_bar(self):
+        w = Window(title="foo")
+        w.use_header_bar()
+        self.assertEqual(w.get_title(), "foo")
+
+        w = Window()
+        w.use_header_bar()
+        self.assertEqual(w.get_title(), None)
