@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import shutil
 import os
 from quodlibet import config
@@ -7,7 +8,8 @@ from quodlibet.formats._audio import AudioFile as Fakesong
 from quodlibet.formats._audio import INTERN_NUM_DEFAULT, PEOPLE
 from quodlibet.util.collection import Album, Playlist, avg, bayesian_average
 from quodlibet.library.libraries import FileLibrary
-from quodlibet.util import format_rating, fsnative
+from quodlibet.util import format_rating
+from quodlibet.util.path import fsnative
 
 config.RATINGS = config.HardCodedRatingsPrefs()
 
