@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -46,4 +47,20 @@ class TWindows(TestCase):
         d = windows.get_link_target(path)
         # the second char is only not in latin-1
         self.assertEqual(os.path.basename(d), u"\xe1??.txt")
-        self.assertTrue(isinstance(d, unicode))
\ No newline at end of file
+        self.assertTrue(isinstance(d, unicode))
+
+    def test_environ(self):
+        env = windows.WindowsEnviron()
+        len_ = len(env)
+        env[u"FOO"] = u"bar"
+        self.assertEqual(len(env), len_ + 1)
+        self.assertEqual(env.get(u"FOO"), u"bar")
+        self.assertTrue("FOO" in repr(env))
+        self.assertEqual(len(list(env)), len(env))
+        del env[u"FOO"]
+
+    def test_environ_ascii(self):
+        env = windows.WindowsEnviron()
+        env["FOO"] = u"bar"
+        env["FOO"]
+        del env["FOO"]
